#ifndef CHOMIK_COMPILER_H
#define CHOMIK_COMPILER_H

#include <chomik.h>
#include "parser.tab.h"
#include "llvm/ADT/APFloat.h"
#include "llvm/ADT/STLExtras.h"
#include "llvm/IR/BasicBlock.h"
#include "llvm/IR/Constants.h"
#include "llvm/IR/DerivedTypes.h"
#include "llvm/IR/Function.h"
#include "llvm/IR/IRBuilder.h"
#include "llvm/IR/LLVMContext.h"
#include "llvm/IR/Module.h"
#include "llvm/IR/Type.h"
#include "llvm/IR/Verifier.h"
#include "llvm/IR/LegacyPassManager.h"
#include "llvm/ADT/StringRef.h"
#include "llvm/ExecutionEngine/JITSymbol.h"
#include "llvm/ExecutionEngine/Orc/CompileUtils.h"
#include "llvm/ExecutionEngine/Orc/Core.h"
#include "llvm/ExecutionEngine/Orc/ExecutionUtils.h"
#include "llvm/ExecutionEngine/Orc/ExecutorProcessControl.h"
#include "llvm/ExecutionEngine/Orc/IRCompileLayer.h"
#include "llvm/ExecutionEngine/Orc/JITTargetMachineBuilder.h"
#include "llvm/ExecutionEngine/Orc/RTDyldObjectLinkingLayer.h"
#include "llvm/ExecutionEngine/Orc/Shared/ExecutorSymbolDef.h"
#include "llvm/ExecutionEngine/SectionMemoryManager.h"
#include "llvm/Passes/PassBuilder.h"
#include "llvm/Passes/StandardInstrumentations.h"
#include "llvm/Support/TargetSelect.h"
#include "llvm/Target/TargetMachine.h"
#include "llvm/Transforms/InstCombine/InstCombine.h"
#include "llvm/Transforms/Scalar.h"
#include "llvm/Transforms/Scalar/GVN.h"
#include "llvm/Transforms/Scalar/Reassociate.h"
#include "llvm/Transforms/Scalar/SimplifyCFG.h"
#include "llvm/MC/TargetRegistry.h"
#include "llvm/Support/FileSystem.h"
#include "llvm/Support/raw_ostream.h"
#include "llvm/Target/TargetMachine.h"
#include "llvm/Target/TargetOptions.h"
#include "llvm/TargetParser/Host.h"

using namespace llvm;



namespace chomik_compiler
{
    namespace AST
    {
        class variable_with_valueAST
        {
        public:
            virtual ~variable_with_valueAST() = default;
            virtual Value *codegen() = 0;
        };

        class placeholder_with_valueAST
        {
        public:
            virtual ~placeholder_with_valueAST() = default;
            virtual Value *codegen() = 0;
        };

        class signature_itemAST
        {
        public:
            virtual ~signature_itemAST() = default;
            virtual Value *codegen() = 0;
        };

        class simple_value_integer_signature_itemAST: public signature_itemAST
        {
        public:
            virtual ~simple_value_integer_signature_itemAST() = default;
            virtual Value *codegen() override;
        };

        class simple_value_float_signature_itemAST: public signature_itemAST
        {
        private:
            double value;
        public:
            simple_value_float_signature_itemAST(double v): value{v} {}
            virtual ~simple_value_float_signature_itemAST() = default;
            virtual Value *codegen() override;
        };

        class simple_value_string_signature_itemAST: public signature_itemAST
        {
        public:
            virtual ~simple_value_string_signature_itemAST() = default;
            virtual Value *codegen() override;
        };

        class simple_value_enum_signature_itemAST: public signature_itemAST
        {
        public:
            virtual ~simple_value_enum_signature_itemAST() = default;
            virtual Value *codegen() override;
        };

        class code_signature_itemAST: public signature_itemAST
        {
        public:
            virtual ~code_signature_itemAST() = default;
            virtual Value *codegen() override;
        };

        class signatureAST
        {
        public:
            virtual ~signatureAST() = default;
            virtual Value *codegen() = 0;
        };


        class name_itemAST
        {
        public:
            virtual ~name_itemAST() = default;
            virtual Value *codegen() = 0;
        };

        class name_item_placeholderAST: public name_itemAST
        {
        public:
            virtual ~name_item_placeholderAST() = default;
            virtual Value *codegen() override;
        };

        class identifier_name_itemAST: public name_itemAST
        {
        private:
            const std::string identifier;
        public:
            identifier_name_itemAST(const std::string & s): identifier{s} {}
            virtual ~identifier_name_itemAST() = default;
            virtual Value *codegen() override;
        };

        class int_literal_name_itemAST: public name_itemAST
        {
        private:
            int value;
        public:
            int_literal_name_itemAST(int v): value{v} {}
            virtual ~int_literal_name_itemAST() = default;
            virtual Value *codegen() override;
        };

        class float_literal_name_itemAST: public name_itemAST
        {
        private:
            double value;
        public:
            float_literal_name_itemAST(double v): value{v} {}
            virtual ~float_literal_name_itemAST() = default;
            virtual Value *codegen() override;
        };

        class string_literal_name_itemAST: public name_itemAST
        {
        private:
            const std::string value;
        public:
            string_literal_name_itemAST(const std::string & s): value{s} {}
            virtual ~string_literal_name_itemAST() = default;
            virtual Value *codegen() override;
        };

        class nonempty_list_of_name_itemsAST
        {
        public:
            std::vector<std::unique_ptr<name_itemAST>> my_vector;

            virtual ~nonempty_list_of_name_itemsAST() = default;
            virtual Value *codegen();
        };

        class code_literal_name_itemAST: public name_itemAST
        {
        public:
            virtual ~code_literal_name_itemAST() = default;
            virtual Value *codegen() override;
        };

        class signature_regular_expressionAST
        {
        public:
            virtual ~signature_regular_expressionAST() = default;
            virtual Value *codegen();
        };

        class generic_typeAST
        {
        public:
            virtual ~generic_typeAST() = default;
            virtual Value *codegen() = 0;
        };

        class generic_type_namedAST: public generic_typeAST
        {
        public:
            virtual ~generic_type_namedAST() = default;
            virtual Value *codegen() override;
        };

        class statementAST
        {
        public:
            virtual ~statementAST() = default;
            virtual Value *codegen() = 0;
        };

        class assignment_statementAST: public statementAST
        {
        public:
            virtual ~assignment_statementAST() = default;
            virtual Value *codegen() override;
        };

        class variable_execution_statementAST: public statementAST
        {
        private:
            std::vector<std::unique_ptr<name_itemAST>> my_vector;
        public:
            variable_execution_statementAST(std::vector<std::unique_ptr<name_itemAST>> && v):
                my_vector{std::move(v)} {}


            virtual ~variable_execution_statementAST() = default;
            virtual Value *codegen() override;
        };
    }

    class parser: public chomik::parser
    {
    public:
        static Function* TheFunction;
        static Value* TheChomikMachine;
    private:
        static int token;
        static YYSTYPE lvalue;

        static int get_line_number();
        static int get_next_token();

        static std::unique_ptr<AST::nonempty_list_of_name_itemsAST> parse_nonempty_list_of_name_items(int first_token);
        static std::unique_ptr<AST::code_literal_name_itemAST> parse_code_literal_name_item(int first_token);
        static std::unique_ptr<AST::name_item_placeholderAST> parse_name_item_placeholder(int first_token);
        static std::unique_ptr<AST::name_itemAST> parse_name_item(int first_token);
        static std::unique_ptr<AST::assignment_statementAST> parse_assignment_statement(int first_token);
        static std::unique_ptr<AST::variable_execution_statementAST> parse_variable_execution_statement(int first_token);
        static std::unique_ptr<AST::statementAST> parse_statement(int first_token);

        static Function* MainLoop();

    public:
        parser(chomik::program & p): chomik::parser{p} {}

        int parse(const char * filename);
    };



    template <typename ABSTRACT_SYNTAX_TREE_TYPE>
    std::unique_ptr<ABSTRACT_SYNTAX_TREE_TYPE> LogError(const char *Str)
    {
        std::cerr << "error" << Str << "\n";
        return nullptr;
    }

}

#endif
